package com.mgh.pmdm.parques.view.ui

import android.os.Bundle
import androidx.fragment.app.Fragment
import android.view.LayoutInflater
import android.view.View
import android.view.ViewGroup
import android.widget.Toast
import androidx.core.os.bundleOf
import androidx.lifecycle.Observer
import androidx.lifecycle.ViewModelProvider
import androidx.navigation.fragment.findNavController
import androidx.recyclerview.widget.LinearLayoutManager
import com.mgh.pmdm.parques.view.dialogs.MiDialogFragment
import com.mgh.pmdm.parques.model.Park
import com.mgh.pmdm.parques.R
import com.mgh.pmdm.parques.databinding.FragmentFirstBinding
import com.mgh.pmdm.parques.viewmodel.FirstFragmentViewModel

class FirstFragment : Fragment(), MiDialogFragment.OnOKOrCancelListener {

    // Propiedad privada para almacenar temporalmente un item
    // se utiliza cuando se va a eliminar un parque
    private var itemToRemove: Park?

    init {
        itemToRemove = null
    }

    // Definimos una instancia del ViewModel como lateinit
    private lateinit var firstFragmentViewModel: FirstFragmentViewModel

    // Definimos el data binding con la posibilidad de que
    // tenga valores nulos, y establecemos el getter
    private var _binding: FragmentFirstBinding? = null
    private val binding get() = _binding!!

    // El método onCreate se ha eliminado, ya que la clase
    // Parks no se inicializa ahora desde aquí.

    override fun onCreateView(
        inflater: LayoutInflater, container: ViewGroup?,
        savedInstanceState: Bundle?
    ): View? {

        // Inyectamos la interfaz a través del Data Binding
        _binding = FragmentFirstBinding.inflate(inflater, container, false)

        return binding.root

    }

    override fun onViewCreated(view: View, savedInstanceState: Bundle?) {
        super.onViewCreated(view, savedInstanceState)

        // En primer lugar, instanciamos el ViewModel, utilizando la clase ViewModelProvider
        firstFragmentViewModel =
            ViewModelProvider(this).get(FirstFragmentViewModel::class.java)

        // Preparación del RecyclerView:

        // 1. Asociamos el LayoutManager
        binding.ParksRecyclerView.layoutManager = LinearLayoutManager(requireActivity())

        // 2. Creamos un observador y lo suscribimos al LiveData adaptador
        //    definido en el ViewModel. De este modo, cuando se produzcan cambios
        //    en el ViewModel, se reflejaran en el adaptador del RecyclerView
        //    Nota: El Observer será el de la clase androidx.lifecycle

        firstFragmentViewModel.adaptador.observe(viewLifecycleOwner, {
            binding.ParksRecyclerView.adapter = it
        })

        // 3. Creamos otro observer y lo susbribimos a los cambios
        //    que se produzcan en el objeto parkClicked del ViewModel.
        //    En este caso, para crear el observer, se recibe el
        //    parque sobre el que se ha hecho click.
        firstFragmentViewModel.parkClicked.observe(viewLifecycleOwner,  { park ->
            park?.let {
                // Reseteamos el valor de parkClicked
                firstFragmentViewModel.parkClicked.value = null

                // Y lanzmos la navegación hacia el fragmento de edición
                val bundle = bundleOf("park" to park)
                findNavController().navigate(R.id.action_FirstFragment_to_SecondFragment, bundle)
            }
        })

        // Creamos un tercer Observer y lo susbribimos a los cambios que se produzcan
        // en el objeto parkLongClicked del homeViewModel, igual que en el caso anterior
        firstFragmentViewModel.parkLongClicked.observe(viewLifecycleOwner,  { park ->
            park?.let {
                // Reseteamos el valor
                firstFragmentViewModel.parkLongClicked.value = null

                // lanzamos la acción correspondiente al click largo, en este caso
                // mostrar el diálogo de confirmación para eliminar el elemento.
                // La vista sigue implementando la interfaz MiDialogFragment.OnOKOrCancelListener
                // por lo que implementará el método onPositiveClick para cuando
                // se haga clic en aceptar en el diálogo.
                // Dentro de este callback, se invocará al método removePark
                // del propio ViewModel, que será el que acceda al modelo
                // a través del repositorio para eliminar el parque en cuestión.

                // Establecemos el elemento a eliminar
                itemToRemove = park

                // e Invocamos al diálogo para preguntar si deseamos eliminar el registro
                val miDialogo = MiDialogFragment(
                    resources.getString(R.string.deletePark),
                    resources.getString(R.string.askDeletePark) + park.name + "?",
                    this
                )

                miDialogo.show(requireActivity().supportFragmentManager, "miDialogo")

            }

        })

        /* -> En el Viewmodel
        // 2. Determinamos que tiene tamaño fijo (optimización)
        binding.ParksRecyclerView.setHasFixedSize(true)
        // 3. Asignamos el adaptador al RecyclerView, proporcionándole
        //    dos funciones lambda, que invocarán los callbacks definidos en esta clase,
        //    para gestionar el click y el click largo sobre un item del RecyclerView.
        binding.ParksRecyclerView.adapter = AdaptadorParques(
            { park: Park, v: View -> itemClicked(park, v) },
            { park: Park, v: View -> itemLongClicked(park, v) }
        )*/

        /* Fin del código importado */




    }

    override fun onDestroyView() {
        super.onDestroyView()
        _binding = null
    }

    /* Otros métodos*/



    // Métodos para implementar la interfaz MiDialogFragment.OnOKOrCancelListener
    override fun onPositiveClick() {
        // Si se pulsa en Aceptar en el diálogo
        // eliminamos el item.
        itemToRemove?.also {
            val index = firstFragmentViewModel.removePark(it)
        }
    }

    override fun onCancelClick() {
        // Si pulsa cancelar mostramos un toast
        Toast.makeText(
            //applicationContext,
            requireActivity().applicationContext,
            resources.getString(R.string.actionCancelled), Toast.LENGTH_SHORT
        ).show()
    }

}